@@ -9,6 +9,14 @@ use std::{
     sync::Arc,
 };
 use swc_common::collections::AHashMap;
+use wasmer_cache::FileSystemCache;
+
+/// Type of cache to store compiled bytecodes of plugins.
+/// Currently only supports filesystem, but _may_ supports
+/// other type (in-memory, etcs) for the long-running processes like devserver.
+pub enum PluginCache {
+    File(FileSystemCache),
+}
 
 /// TODO: Cache
 pub fn resolve(name: &str) -> Result<Arc<PathBuf>, Error> {
@@ -44,6 +52,33 @@ pub fn resolve(name: &str) -> Result<Arc<PathBuf>, Error> {
     bail!("failed to resolve plugin `{}`:\n{:?}", name, errors)
 }
 
+/// Build a path to cache location where plugin's bytecode cache will be stored.
+/// This fn does a side effect to create path to cache if given path is not
+/// resolvable. If root is not specified, it'll generate default root for cache
+/// location.
+///
+/// Note SWC's plugin should not fail to load when cache location is not
+/// available. It'll make each invocation to cold start.
+pub fn resolve_plugin_cache_root(root: Option<String>) -> Result<PluginCache, Error> {
+    let root_path = match root {
+        Some(root) => {
+            let mut root = PathBuf::from(root);
+            root.push("plugins");
+            root
+        }
+        None => {
+            let mut cwd = current_dir().context("failed to get current directory")?;
+            cwd.push(".swc");
+            cwd.push("plugins");
+            cwd
+        }
+    };
+
+    FileSystemCache::new(root_path)
+        .map(|cache| PluginCache::File(cache))
+        .context("Failed to create cache location for the plugins")
+}
+
 #[derive(Deserialize)]
 struct PkgJson {
     main: String,
